/* Generated By:JavaCC: Do not edit this line. JsonParser.java */
/*
 * #%L
 * JsonParser.jj - mongodb-async-driver - Allanbank Consulting, Inc.
 * %%
 * Copyright (C) 2011 - 2014 Allanbank Consulting, Inc.
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
package com.allanbank.mongodb.bson.json;

import java.io.Reader;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;

import com.allanbank.mongodb.bson.Document;
import com.allanbank.mongodb.bson.Element;
import com.allanbank.mongodb.bson.builder.ArrayBuilder;
import com.allanbank.mongodb.bson.builder.BuilderFactory;
import com.allanbank.mongodb.bson.builder.DocumentBuilder;
import com.allanbank.mongodb.bson.element.ObjectId;
import com.allanbank.mongodb.error.JsonParseException;
import com.allanbank.mongodb.bson.io.EndianUtils;
import com.allanbank.mongodb.util.IOUtils;

/**
 * Parser for JSON documents.  This parser does not handle functions or complex expressions.
 * <p>
 * The intent of this parser is not to create a full capability JSON parser but 
 * instead to provide a simplified parser that accepts most valid documents.  
 * This parsers may also accept invalid documents/programs.
 * </p>
 * <p>
 * This class is not currently part of the drivers API.  We intend to eventually provide some 
 * basic JSON-->Document functionality but are looking for feedback from the community
 * on that interface. 
 * </p>
 * <p>
 * Users should not utilize this class directly.  Instead they should use the 
 * {@link Json} static class.
 * </p>
 *
 * @see <a href="http://www.ietf.org/rfc/rfc4627.txt">RFC 4627</a>
 * @api.no This class is <b>NOT</b> part of the drivers API. This class may be
 *         mutated in incompatible ways between any two releases of the driver.
 * @copyright 2012-2013, Allanbank Consulting, Inc., All Rights Reserved
 */
@SuppressWarnings({ "unused", "javadoc", "deprecated" })
/* package */ class JsonParser implements JsonParserConstants {
    /** The default time zone. */
    public static final TimeZone UTC = TimeZone.getTimeZone("UTC");

    /** The time formats accepted/parsed. */
    private static final String[] DATE_FORMATS = new String[] {
            "yyyy-MM-dd'T'HH:mm:ss.SSSZ", "yyyy-MM-dd'T'HH:mm:ssZ",
            "yyyy-MM-dd'T'HH:mmZ", "yyyy-MM-dd'T'HH:mm:ss.SSS",
            "yyyy-MM-dd'T'HH:mm:ss", "yyyy-MM-dd'T'HH:mm",
            "yyyy-MM-dd HH:mm:ss.SSSZ", "yyyy-MM-dd HH:mm:ssZ",
            "yyyy-MM-dd HH:mmZ", "yyyy-MM-dd HH:mm:ss.SSS",
            "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM-dd", };

    /**
     * Create a new JsonParser.
     */
    public JsonParser() {
        this(new StringReader(""));
    }

    /**
     * Parses the current source of data.
     * 
     * @param text
     *            The text JSON to parse.
     * @return The Document or list of documents parsed.
     * @throws ParseException
     *             On a failure to parse the JSON document.
     */
    public Object parse(final String text) throws ParseException {
        ReInit(new StringReader(text));
        return parse();
    }

    /**
     * Parses the current source of data.
     * 
     * @param reader
     *            The text JSON to parse.
     * @return The Document or list of documents parsed.
     * @throws ParseException
     *             On a failure to parse the JSON document.
     */
    public Object parse(final Reader reader) throws ParseException {
        ReInit(reader);
        return parse();
    }

    /**
     * Converts the Base64 string to a binary array.
     * 
     * @param base64Token
     *            The Base64 encoded bytes.
     * @return The decoded bytes.
     * @throws JsonParseException
     *             On a failure to decodee the binary.
     */
    private byte[] toBinaryFromBase64(final Token base64Token)
            throws JsonParseException {
        final String base64 = trimQuotes(base64Token.image);
        try {
            return IOUtils.base64ToBytes(base64);
        }
        catch (final IllegalArgumentException iae) {
            throw new JsonParseException(iae.getMessage() + " (line "
                    + base64Token.beginLine + ", column "
                    + base64Token.beginColumn + ")", iae,
                    base64Token.beginLine, base64Token.beginColumn);
        }
    }

    /**
     * Converts the HEX string to a binary array.
     * 
     * @param hexToken
     *            The HEX encoded bytes.
     * @return The decoded bytes.
     * @throws JsonParseException
     *             On a failure to decodee the binary.
     */
    private byte[] toBinaryFromHex(final Token hexToken)
            throws JsonParseException {
        final String hex = trimQuotes(hexToken.image);
        try {
            return IOUtils.hexToBytes(hex);
        }
        catch (final IllegalArgumentException iae) {
            throw new JsonParseException(iae.getMessage() + " (line "
                    + hexToken.beginLine + ", column " + hexToken.beginColumn
                    + ")", iae, hexToken.beginLine, hexToken.beginColumn);
        }
    }

    /**
     * Parses the ISO date string into a long field.
     * 
     * @param timestampToken
     *            The timestamp string.
     * @return The Date for the time string.
     * @throws JsonParseException
     *             On a failure to parse the date string.
     */
    private Date toDate(final Token timestampToken) throws JsonParseException {

        Exception last = null;
        final String timestamp = trimQuotes(timestampToken.image);
        for (final String format : DATE_FORMATS) {
            final SimpleDateFormat f = new SimpleDateFormat(format);
            f.setTimeZone(UTC);
            try {
                return f.parse(timestamp);
            }
            catch (final java.text.ParseException pe) {
                // Ignore. Try the next.
                last = pe;
            }
        }

        throw new JsonParseException("Could not parse the time string '"
                + timestamp + "' @ line " + timestampToken.beginLine
                + ", column " + timestampToken.beginColumn + ".", last,
                timestampToken.beginLine, timestampToken.beginColumn);
    }

    /**
     * Parses the ISO date string into a long field.
     * 
     * @param timestampToken
     *            The timestamp string.
     * @return The Date for the time string.
     * @throws JsonParseException
     *             On a failure to parse the date string.
     */
    private Date toDateFromEpoch(final Token timestampToken)
            throws JsonParseException {

        try {
            return new Date(Long.parseLong(timestampToken.image));
        }
        catch (NumberFormatException nfe) {
            throw new JsonParseException("Could not parse the time milliseconds '"
                    + timestampToken.image + "' @ line " + timestampToken.beginLine
                    + ", column " + timestampToken.beginColumn + ".", nfe,
                    timestampToken.beginLine, timestampToken.beginColumn);
        }
    }

    /**
     * Converts the HEX string to an integer.
     * 
     * @param hexToken
     *            The HEX encoded bytes.
     * @return The decoded integer.
     * @throws JsonParseException
     *             On a failure to decodee the integer.
     */
    private int toIntFromHex(final Token hexToken) throws JsonParseException {
        String hex = trimQuotes(hexToken.image);
        if (hex.startsWith("0x") || hex.startsWith("0X")) {
            hex = hex.substring(2);
        }

        try {
            return Integer.parseInt(hex, 16);
        }
        catch (final NumberFormatException nfe) {
            throw new JsonParseException(nfe.getMessage() + " (line "
                    + hexToken.beginLine + ", column " + hexToken.beginColumn
                    + ")", nfe, hexToken.beginLine, hexToken.beginColumn);
        }
    }

    /**
     * Converts the timestamp and increment into a MongoTimestamp Long value..
     * 
     * @param timestampToken
     *            The Mongo timestamp value (seconds since UNIX Epoch).
     * @param incrementToken
     *            The increment for the Mongo Timestamp.
     * @return The MongoTimestamp long value.
     * @throws JsonParseException
     *             On a failure to decode the timestamp integers.
     */
    private long toMongoTimestamp(final Token timestampToken,
            final Token incrementToken) throws JsonParseException {
        long value = 0;

        final String timestamp = trimQuotes(timestampToken.image);
        final String increment = trimQuotes(incrementToken.image);
        try {
            final long timePortion = Long.parseLong(timestamp) & 0xFFFFFFFFL;

            // Time is specified in milliseconds but only store seconds
            // so we truncate the time to milliseconds.
            value = TimeUnit.MILLISECONDS.toSeconds(timePortion);
            value <<= Integer.SIZE;
            value += (Long.parseLong(increment) & 0xFFFFFFFFL);
        }
        catch (final NumberFormatException nfe) {
            throw new JsonParseException(nfe.getMessage() + " (line "
                    + timestampToken.beginLine + ", column "
                    + timestampToken.beginColumn + ")", nfe,
                    timestampToken.beginLine, timestampToken.beginColumn);
        }
        return value;
    }

    /**
     * Tries to create a Mongo Timestamp from the four tokens in a document that
     * looks like:
     * <code>{ $timestamp : { &lt;token1&gt; : &lt;token2&gt; , &lt;token3&gt; : &lt;token4&gt; } }</code>
     * The second and fourth tokens should be integers. The first and third
     * should be either 't' or 'i'.
     * 
     * @param t1
     *            The first token in the document.
     * @param t2
     *            The second token in the document, should be an integer.
     * @param t3
     *            The third token in the document.
     * @param t4
     *            The fourth token in the document, should be an integer.
     */
    private long toMongoTimestamp(Token t1, Token t2, Token t3, Token t4) {
        try {
            long t2Int = Long.parseLong(trimQuotes(t2.image));
            long t4Int = Long.parseLong(trimQuotes(t4.image));

            String t1Str = trimQuotes(t1.image);
            String t3Str = trimQuotes(t3.image);

            long value = 0;
            if ("t".equals(t1Str) && "i".equals(t3Str)) {
                // Time is specified in milliseconds but only store seconds
                // so we truncate the time to milliseconds.
                value = TimeUnit.MILLISECONDS.toSeconds(t2Int);
                value <<= Integer.SIZE;
                value += (t4Int & 0xFFFFFFFFL);
            }
            else if ("t".equals(t3Str) && "i".equals(t1Str)) {
                value = TimeUnit.MILLISECONDS.toSeconds(t4Int);
                value <<= Integer.SIZE;
                value += (t2Int & 0xFFFFFFFFL);
            }
            else {
                throw new JsonParseException(
                        "Invalid MongoDB Timestamp document at line "
                                + t1.beginLine + ", column " + t1.beginColumn
                                + ":", t1.beginLine, t1.beginColumn);
            }
            return value;
        }
        catch (NumberFormatException nfe) {
            throw new JsonParseException(nfe.getMessage() + " (line "
                    + t1.beginLine + ", column " + t1.beginColumn + ")", nfe,
                    t1.beginLine, t1.beginColumn);
        }
    }

    /**
     * Creates a ObjectId from the hex binary representation.
     * 
     * @param hexBytesToken
     *            The hex bytes for the ObjectId.
     * @return The ObjectId.
     * @throws JsonParseException
     *             On a failure to decode the hex binary into an ObjectId.
     */
    private ObjectId toObjectId(final Token hexBytesToken)
            throws JsonParseException {
        int timestamp = 0;
        long machineId = 0;

        final String hexBytes = trimQuotes(hexBytesToken.image);
        try {
            return new ObjectId(hexBytes);
        }
        catch (final IllegalArgumentException iae) {
            throw new JsonParseException(iae.getMessage() + " (line "
                    + hexBytesToken.beginLine + ", column "
                    + hexBytesToken.beginColumn + ")", iae,
                    hexBytesToken.beginLine, hexBytesToken.beginColumn);
        }
    }

    /**
     * Trims the quotes from the string.
     * 
     * @param string
     *            The string to trim the quotes from.
     * @return The string without the quotes.
     */
    private String trimQuotes(final String string) {
        if (!string.isEmpty()) {
            final int length = string.length();
            final char first = string.charAt(0);
            final char last = string.charAt(length - 1);
            if ((first == last) && ((first == '"') || (first == '\u005c''))) {
                return string.substring(1, length - 1);
            }
        }

        return string;
    }

  final protected Object parse() throws ParseException {
    Object result = null;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case TOKEN_OPEN_BRACE:
      result = document(null);
      break;
    case TOKEN_OPEN_BRACKET:
      result = array(null);
      break;
    default:
      jj_la1[0] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
        {if (true) return result;}
    throw new Error("Missing return statement in function");
  }

  final private Document document(DocumentBuilder builder) throws ParseException {
    DocumentBuilder b = builder;
    if( b == null ) {
        b = BuilderFactory.start();
    }
    jj_consume_token(TOKEN_OPEN_BRACE);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case STRING_LITERAL:
    case HEX_DIGIT:
    case IDENTIFIER_NAME:
      members(b);
      break;
    default:
      jj_la1[1] = jj_gen;
      ;
    }
    jj_consume_token(TOKEN_CLOSE_BRACE);
        {if (true) return b.build();}
    throw new Error("Missing return statement in function");
  }

  final private List<Element> array(ArrayBuilder builder) throws ParseException {
    ArrayBuilder b = builder;
    if( b == null ) {
        b = BuilderFactory.startArray();
    }
    jj_consume_token(TOKEN_OPEN_BRACKET);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case TOKEN_TRUE:
    case TOKEN_FALSE:
    case TOKEN_NULL:
    case TOKEN_OPEN_BRACE:
    case TOKEN_OPEN_BRACKET:
    case DOUBLE:
    case INTEGER:
    case STRING_LITERAL:
    case HEX_DIGIT:
    case TOKEN_BINDATA:
    case TOKEN_HEXDATA:
    case TOKEN_ISODATE:
    case TOKEN_NUMBERLONG:
    case TOKEN_OBJECTID:
    case TOKEN_TIMESTAMP:
    case TOKEN_MAXKEY:
    case TOKEN_MINKEY:
    case TOKEN_DB_POINTER:
    case IDENTIFIER_NAME:
      elements(b);
      break;
    default:
      jj_la1[2] = jj_gen;
      ;
    }
    jj_consume_token(TOKEN_CLOSE_BRACKET);
        {if (true) return Arrays.asList(b.build());}
    throw new Error("Missing return statement in function");
  }

  final private void members(DocumentBuilder builder) throws ParseException {
    pair(builder);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case TOKEN_COMMA:
      jj_consume_token(TOKEN_COMMA);
      members(builder);
      break;
    default:
      jj_la1[3] = jj_gen;
      ;
    }
  }

  final private void pair(DocumentBuilder builder) throws ParseException {
    Token nameToken = null;
    String name = null;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case IDENTIFIER_NAME:
      nameToken = jj_consume_token(IDENTIFIER_NAME);
                                          name = nameToken.image;
      break;
    case STRING_LITERAL:
      nameToken = jj_consume_token(STRING_LITERAL);
                                         name = trimQuotes( nameToken.image );
      break;
    case HEX_DIGIT:
      nameToken = jj_consume_token(HEX_DIGIT);
                                    name = nameToken.image;
      break;
    default:
      jj_la1[4] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    jj_consume_token(TOKEN_COLON);
    documentValue(name, builder);
  }

  final private void elements(ArrayBuilder builder) throws ParseException {
    arrayValue(builder);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case TOKEN_COMMA:
      jj_consume_token(TOKEN_COMMA);
      elements(builder);
      break;
    default:
      jj_la1[5] = jj_gen;
      ;
    }
  }

  final private void arrayValue(ArrayBuilder builder) throws ParseException {
    Token t1 = null;
    Token t2 = null;
    Token t3 = null;
    Token t4 = null;
    if (jj_2_1(2)) {
      jj_consume_token(TOKEN_OPEN_BRACE);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case TOKEN_BINARY:
        jj_consume_token(TOKEN_BINARY);
        jj_consume_token(TOKEN_COLON);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(TOKEN_COMMA);
        jj_consume_token(TOKEN_TYPE);
        jj_consume_token(TOKEN_COLON);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case INTEGER:
          t1 = jj_consume_token(INTEGER);
          break;
        case STRING_LITERAL:
          t3 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[6] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
              if( t1 != null ) {
                  builder.addBinary( (byte) Integer.parseInt(t1.image), toBinaryFromBase64(t2) );
              } else {
                  builder.addBinary( (byte) toIntFromHex(t3), toBinaryFromBase64(t2) );
              }
        break;
      case TOKEN_TYPE:
        jj_consume_token(TOKEN_TYPE);
        jj_consume_token(TOKEN_COLON);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case INTEGER:
          t1 = jj_consume_token(INTEGER);
          break;
        case STRING_LITERAL:
          t3 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[7] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
        jj_consume_token(TOKEN_COMMA);
        jj_consume_token(TOKEN_BINARY);
        jj_consume_token(TOKEN_COLON);
        t2 = jj_consume_token(STRING_LITERAL);
              if( t1 != null ) {
                  builder.addBinary( (byte) Integer.parseInt(t1.image), toBinaryFromBase64(t2) );
              } else {
                  builder.addBinary( (byte) toIntFromHex(t3), toBinaryFromBase64(t2) );
              }
        break;
      case TOKEN_DATE:
        jj_consume_token(TOKEN_DATE);
        jj_consume_token(TOKEN_COLON);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case INTEGER:
          t1 = jj_consume_token(INTEGER);
          break;
        case STRING_LITERAL:
          t2 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[8] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
              if ( t1 != null ) {
                  builder.add( toDateFromEpoch(t1) );
              } else {
                  builder.add( toDate(t2) );
              }
        break;
      case TOKEN_TS:
        jj_consume_token(TOKEN_TS);
        jj_consume_token(TOKEN_COLON);
        jj_consume_token(TOKEN_OPEN_BRACE);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case IDENTIFIER_NAME:
          t1 = jj_consume_token(IDENTIFIER_NAME);
          break;
        case STRING_LITERAL:
          t1 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[9] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
        jj_consume_token(TOKEN_COLON);
        t2 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_COMMA);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case IDENTIFIER_NAME:
          t3 = jj_consume_token(IDENTIFIER_NAME);
          break;
        case STRING_LITERAL:
          t3 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[10] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
        jj_consume_token(TOKEN_COLON);
        t4 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_CLOSE_BRACE);
              builder.addMongoTimestamp( toMongoTimestamp(t1, t2, t3, t4) );
        break;
      case TOKEN_REGEX:
        jj_consume_token(TOKEN_REGEX);
        jj_consume_token(TOKEN_COLON);
        t1 = jj_consume_token(STRING_LITERAL);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case TOKEN_COMMA:
          jj_consume_token(TOKEN_COMMA);
          jj_consume_token(TOKEN_OPTIONS);
          jj_consume_token(TOKEN_COLON);
          t2 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[11] = jj_gen;
          ;
        }
              if ( t2 != null ) {
                  builder.addRegularExpression( trimQuotes(t1.image), trimQuotes(t2.image) );
              } else {
                  builder.addRegularExpression( trimQuotes(t1.image), "" );
              }
        break;
      case TOKEN_OPTIONS:
        jj_consume_token(TOKEN_OPTIONS);
        jj_consume_token(TOKEN_COLON);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(TOKEN_COMMA);
        jj_consume_token(TOKEN_REGEX);
        jj_consume_token(TOKEN_COLON);
        t1 = jj_consume_token(STRING_LITERAL);
              builder.addRegularExpression( trimQuotes(t1.image), trimQuotes(t2.image) );
        break;
      case TOKEN_OID:
        jj_consume_token(TOKEN_OID);
        jj_consume_token(TOKEN_COLON);
        t1 = jj_consume_token(STRING_LITERAL);
              builder.addObjectId( toObjectId(t1) );
        break;
      case TOKEN_MAX:
        jj_consume_token(TOKEN_MAX);
        jj_consume_token(TOKEN_COLON);
        jj_consume_token(INTEGER);
             builder.addMaxKey();
        break;
      case TOKEN_MIN:
        jj_consume_token(TOKEN_MIN);
        jj_consume_token(TOKEN_COLON);
        jj_consume_token(INTEGER);
             builder.addMinKey();
        break;
      default:
        jj_la1[12] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      jj_consume_token(TOKEN_CLOSE_BRACE);
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case TOKEN_OPEN_BRACE:
        document(builder.push());
        break;
      case TOKEN_OPEN_BRACKET:
        array(builder.pushArray());
        break;
      case HEX_DIGIT:
        t1 = jj_consume_token(HEX_DIGIT);
                             builder.addSymbol(t1.image);
        break;
      case IDENTIFIER_NAME:
        t1 = jj_consume_token(IDENTIFIER_NAME);
                                   builder.addSymbol(t1.image);
        break;
      case STRING_LITERAL:
        t1 = jj_consume_token(STRING_LITERAL);
                                  builder.addString(trimQuotes(t1.image));
        break;
      case DOUBLE:
        t1 = jj_consume_token(DOUBLE);
                          builder.addDouble(Double.parseDouble(t1.image));
        break;
      case INTEGER:
        t1 = jj_consume_token(INTEGER);
             // Experimentally determine the size of the integer.
             long value = Long.parseLong(t1.image);
             if( (Integer.MIN_VALUE <= value) && (value <= Integer.MAX_VALUE) ) {
                builder.addInteger((int) value);
             } else {
                builder.addLong(value);
             }
        break;
      case TOKEN_TRUE:
        jj_consume_token(TOKEN_TRUE);
                         builder.addBoolean(true);
        break;
      case TOKEN_FALSE:
        jj_consume_token(TOKEN_FALSE);
                          builder.addBoolean(false);
        break;
      case TOKEN_NULL:
        jj_consume_token(TOKEN_NULL);
                         builder.addNull();
        break;
      case TOKEN_BINDATA:
        jj_consume_token(TOKEN_BINDATA);
        jj_consume_token(69);
        t1 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_COMMA);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.addBinary( (byte) Integer.parseInt(t1.image), toBinaryFromBase64(t2) );
        break;
      case TOKEN_HEXDATA:
        jj_consume_token(TOKEN_HEXDATA);
        jj_consume_token(69);
        t1 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_COMMA);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.addBinary( (byte) Integer.parseInt(t1.image), toBinaryFromHex(t2) );
        break;
      case TOKEN_ISODATE:
        jj_consume_token(TOKEN_ISODATE);
        jj_consume_token(69);
        t1 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.add( toDate(t1) );
        break;
      case TOKEN_NUMBERLONG:
        jj_consume_token(TOKEN_NUMBERLONG);
        jj_consume_token(69);
        t1 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.addLong(Long.parseLong(trimQuotes(t1.image)));
        break;
      case TOKEN_OBJECTID:
        jj_consume_token(TOKEN_OBJECTID);
        jj_consume_token(69);
        t1 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.addObjectId( toObjectId(t1) );
        break;
      case TOKEN_TIMESTAMP:
        jj_consume_token(TOKEN_TIMESTAMP);
        jj_consume_token(69);
        t1 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_COMMA);
        t2 = jj_consume_token(INTEGER);
        jj_consume_token(70);
             builder.addMongoTimestamp( toMongoTimestamp(t1, t2) );
        break;
      case TOKEN_MAXKEY:
        jj_consume_token(TOKEN_MAXKEY);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case 69:
          jj_consume_token(69);
          jj_consume_token(70);
          break;
        default:
          jj_la1[13] = jj_gen;
          ;
        }
             builder.addMaxKey();
        break;
      case TOKEN_MINKEY:
        jj_consume_token(TOKEN_MINKEY);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case 69:
          jj_consume_token(69);
          jj_consume_token(70);
          break;
        default:
          jj_la1[14] = jj_gen;
          ;
        }
             builder.addMinKey();
        break;
      case TOKEN_DB_POINTER:
        jj_consume_token(TOKEN_DB_POINTER);
        jj_consume_token(69);
        t1 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(TOKEN_COMMA);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(TOKEN_COMMA);
        jj_consume_token(TOKEN_OBJECTID);
        jj_consume_token(69);
        t3 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
        jj_consume_token(70);
             builder.addDBPointer(trimQuotes(t1.image), trimQuotes(t2.image), toObjectId(t3));
        break;
      default:
        jj_la1[15] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final private void documentValue(String name, DocumentBuilder builder) throws ParseException {
    Token t1 = null;
    Token t2 = null;
    Token t3 = null;
    Token t4 = null;
    if (jj_2_2(2)) {
      jj_consume_token(TOKEN_OPEN_BRACE);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case TOKEN_BINARY:
        jj_consume_token(TOKEN_BINARY);
        jj_consume_token(TOKEN_COLON);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(TOKEN_COMMA);
        jj_consume_token(TOKEN_TYPE);
        jj_consume_token(TOKEN_COLON);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case INTEGER:
          t1 = jj_consume_token(INTEGER);
          break;
        case STRING_LITERAL:
          t3 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[16] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
              if( t1 != null ) {
                  builder.addBinary( name, (byte) Integer.parseInt(t1.image), toBinaryFromBase64(t2) );
              } else {
                  builder.addBinary( name, (byte) toIntFromHex(t3), toBinaryFromBase64(t2) );
              }
        break;
      case TOKEN_TYPE:
        jj_consume_token(TOKEN_TYPE);
        jj_consume_token(TOKEN_COLON);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case INTEGER:
          t1 = jj_consume_token(INTEGER);
          break;
        case STRING_LITERAL:
          t3 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[17] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
        jj_consume_token(TOKEN_COMMA);
        jj_consume_token(TOKEN_BINARY);
        jj_consume_token(TOKEN_COLON);
        t2 = jj_consume_token(STRING_LITERAL);
              if( t1 != null ) {
                  builder.addBinary( name, (byte) Integer.parseInt(t1.image), toBinaryFromBase64(t2) );
              } else {
                  builder.addBinary( name, (byte) toIntFromHex(t3), toBinaryFromBase64(t2) );
              }
        break;
      case TOKEN_DATE:
        jj_consume_token(TOKEN_DATE);
        jj_consume_token(TOKEN_COLON);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case INTEGER:
          t1 = jj_consume_token(INTEGER);
          break;
        case STRING_LITERAL:
          t2 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[18] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
              if ( t1 != null ) {
                  builder.add( name, toDateFromEpoch(t1) );
              } else {
                  builder.add( name, toDate(t2) );
              }
        break;
      case TOKEN_TS:
        jj_consume_token(TOKEN_TS);
        jj_consume_token(TOKEN_COLON);
        jj_consume_token(TOKEN_OPEN_BRACE);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case IDENTIFIER_NAME:
          t1 = jj_consume_token(IDENTIFIER_NAME);
          break;
        case STRING_LITERAL:
          t1 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[19] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
        jj_consume_token(TOKEN_COLON);
        t2 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_COMMA);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case IDENTIFIER_NAME:
          t3 = jj_consume_token(IDENTIFIER_NAME);
          break;
        case STRING_LITERAL:
          t3 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[20] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
        jj_consume_token(TOKEN_COLON);
        t4 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_CLOSE_BRACE);
              builder.addMongoTimestamp( name, toMongoTimestamp(t1, t2, t3, t4) );
        break;
      case TOKEN_REGEX:
        jj_consume_token(TOKEN_REGEX);
        jj_consume_token(TOKEN_COLON);
        t1 = jj_consume_token(STRING_LITERAL);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case TOKEN_COMMA:
          jj_consume_token(TOKEN_COMMA);
          jj_consume_token(TOKEN_OPTIONS);
          jj_consume_token(TOKEN_COLON);
          t2 = jj_consume_token(STRING_LITERAL);
          break;
        default:
          jj_la1[21] = jj_gen;
          ;
        }
              if ( t2 != null ) {
                  builder.addRegularExpression( name, trimQuotes(t1.image), trimQuotes(t2.image) );
              } else {
                  builder.addRegularExpression( name, trimQuotes(t1.image), "" );
              }
        break;
      case TOKEN_OPTIONS:
        jj_consume_token(TOKEN_OPTIONS);
        jj_consume_token(TOKEN_COLON);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(TOKEN_COMMA);
        jj_consume_token(TOKEN_REGEX);
        jj_consume_token(TOKEN_COLON);
        t1 = jj_consume_token(STRING_LITERAL);
              builder.addRegularExpression( name, trimQuotes(t1.image), trimQuotes(t2.image) );
        break;
      case TOKEN_OID:
        jj_consume_token(TOKEN_OID);
        jj_consume_token(TOKEN_COLON);
        t1 = jj_consume_token(STRING_LITERAL);
              builder.addObjectId( name, toObjectId(t1) );
        break;
      case TOKEN_MAX:
        jj_consume_token(TOKEN_MAX);
        jj_consume_token(TOKEN_COLON);
        jj_consume_token(INTEGER);
             builder.addMaxKey(name);
        break;
      case TOKEN_MIN:
        jj_consume_token(TOKEN_MIN);
        jj_consume_token(TOKEN_COLON);
        jj_consume_token(INTEGER);
             builder.addMinKey(name);
        break;
      default:
        jj_la1[22] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      jj_consume_token(TOKEN_CLOSE_BRACE);
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case TOKEN_OPEN_BRACE:
        document(builder.push(name));
        break;
      case TOKEN_OPEN_BRACKET:
        array(builder.pushArray(name));
        break;
      case HEX_DIGIT:
        t1 = jj_consume_token(HEX_DIGIT);
                             builder.addSymbol(name, t1.image);
        break;
      case IDENTIFIER_NAME:
        t1 = jj_consume_token(IDENTIFIER_NAME);
                                   builder.addSymbol(name, t1.image);
        break;
      case STRING_LITERAL:
        t1 = jj_consume_token(STRING_LITERAL);
                                  builder.addString(name, trimQuotes(t1.image));
        break;
      case DOUBLE:
        t1 = jj_consume_token(DOUBLE);
                          builder.addDouble(name, Double.parseDouble(t1.image));
        break;
      case INTEGER:
        t1 = jj_consume_token(INTEGER);
             // Experimentally determine the size of the integer.
             long value = Long.parseLong(t1.image);
             if( (Integer.MIN_VALUE <= value) && (value <= Integer.MAX_VALUE) ) {
                builder.addInteger(name, (int) value);
             } else {
                builder.addLong(name, value);
             }
        break;
      case TOKEN_TRUE:
        jj_consume_token(TOKEN_TRUE);
                         builder.addBoolean(name, true);
        break;
      case TOKEN_FALSE:
        jj_consume_token(TOKEN_FALSE);
                          builder.addBoolean(name, false);
        break;
      case TOKEN_NULL:
        jj_consume_token(TOKEN_NULL);
                         builder.addNull(name);
        break;
      case TOKEN_BINDATA:
        jj_consume_token(TOKEN_BINDATA);
        jj_consume_token(69);
        t1 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_COMMA);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.addBinary( name, (byte) Integer.parseInt(t1.image), toBinaryFromBase64(t2) );
        break;
      case TOKEN_HEXDATA:
        jj_consume_token(TOKEN_HEXDATA);
        jj_consume_token(69);
        t1 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_COMMA);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.addBinary( name, (byte) Integer.parseInt(t1.image), toBinaryFromHex(t2) );
        break;
      case TOKEN_ISODATE:
        jj_consume_token(TOKEN_ISODATE);
        jj_consume_token(69);
        t1 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.add( name, toDate(t1) );
        break;
      case TOKEN_NUMBERLONG:
        jj_consume_token(TOKEN_NUMBERLONG);
        jj_consume_token(69);
        t1 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.addLong(name, Long.parseLong(trimQuotes(t1.image)));
        break;
      case TOKEN_OBJECTID:
        jj_consume_token(TOKEN_OBJECTID);
        jj_consume_token(69);
        t1 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
             builder.addObjectId( name, toObjectId(t1) );
        break;
      case TOKEN_TIMESTAMP:
        jj_consume_token(TOKEN_TIMESTAMP);
        jj_consume_token(69);
        t1 = jj_consume_token(INTEGER);
        jj_consume_token(TOKEN_COMMA);
        t2 = jj_consume_token(INTEGER);
        jj_consume_token(70);
             builder.addMongoTimestamp( name, toMongoTimestamp(t1, t2) );
        break;
      case TOKEN_MAXKEY:
        jj_consume_token(TOKEN_MAXKEY);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case 69:
          jj_consume_token(69);
          jj_consume_token(70);
          break;
        default:
          jj_la1[23] = jj_gen;
          ;
        }
             builder.addMaxKey(name);
        break;
      case TOKEN_MINKEY:
        jj_consume_token(TOKEN_MINKEY);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case 69:
          jj_consume_token(69);
          jj_consume_token(70);
          break;
        default:
          jj_la1[24] = jj_gen;
          ;
        }
             builder.addMinKey(name);
        break;
      case TOKEN_DB_POINTER:
        jj_consume_token(TOKEN_DB_POINTER);
        jj_consume_token(69);
        t1 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(TOKEN_COMMA);
        t2 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(TOKEN_COMMA);
        jj_consume_token(TOKEN_OBJECTID);
        jj_consume_token(69);
        t3 = jj_consume_token(STRING_LITERAL);
        jj_consume_token(70);
        jj_consume_token(70);
             builder.addDBPointer(name, trimQuotes(t1.image), trimQuotes(t2.image), toObjectId(t3));
        break;
      default:
        jj_la1[25] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  private boolean jj_2_1(int xla) {
    jj_la = xla; jj_lastpos = jj_scanpos = token;
    try { return !jj_3_1(); }
    catch(LookaheadSuccess ls) { return true; }
    finally { jj_save(0, xla); }
  }

  private boolean jj_2_2(int xla) {
    jj_la = xla; jj_lastpos = jj_scanpos = token;
    try { return !jj_3_2(); }
    catch(LookaheadSuccess ls) { return true; }
    finally { jj_save(1, xla); }
  }

  private boolean jj_3R_5() {
    if (jj_scan_token(TOKEN_REGEX)) return true;
    return false;
  }

  private boolean jj_3R_16() {
    if (jj_scan_token(TOKEN_OID)) return true;
    return false;
  }

  private boolean jj_3_2() {
    if (jj_scan_token(TOKEN_OPEN_BRACE)) return true;
    Token xsp;
    xsp = jj_scanpos;
    if (jj_3R_10()) {
    jj_scanpos = xsp;
    if (jj_3R_11()) {
    jj_scanpos = xsp;
    if (jj_3R_12()) {
    jj_scanpos = xsp;
    if (jj_3R_13()) {
    jj_scanpos = xsp;
    if (jj_3R_14()) {
    jj_scanpos = xsp;
    if (jj_3R_15()) {
    jj_scanpos = xsp;
    if (jj_3R_16()) {
    jj_scanpos = xsp;
    if (jj_3R_17()) {
    jj_scanpos = xsp;
    if (jj_3R_18()) return true;
    }
    }
    }
    }
    }
    }
    }
    }
    return false;
  }

  private boolean jj_3R_15() {
    if (jj_scan_token(TOKEN_OPTIONS)) return true;
    return false;
  }

  private boolean jj_3R_4() {
    if (jj_scan_token(TOKEN_TS)) return true;
    return false;
  }

  private boolean jj_3R_14() {
    if (jj_scan_token(TOKEN_REGEX)) return true;
    return false;
  }

  private boolean jj_3R_3() {
    if (jj_scan_token(TOKEN_DATE)) return true;
    return false;
  }

  private boolean jj_3R_13() {
    if (jj_scan_token(TOKEN_TS)) return true;
    return false;
  }

  private boolean jj_3R_2() {
    if (jj_scan_token(TOKEN_TYPE)) return true;
    return false;
  }

  private boolean jj_3R_9() {
    if (jj_scan_token(TOKEN_MIN)) return true;
    return false;
  }

  private boolean jj_3R_12() {
    if (jj_scan_token(TOKEN_DATE)) return true;
    return false;
  }

  private boolean jj_3R_1() {
    if (jj_scan_token(TOKEN_BINARY)) return true;
    return false;
  }

  private boolean jj_3R_8() {
    if (jj_scan_token(TOKEN_MAX)) return true;
    return false;
  }

  private boolean jj_3R_7() {
    if (jj_scan_token(TOKEN_OID)) return true;
    return false;
  }

  private boolean jj_3R_11() {
    if (jj_scan_token(TOKEN_TYPE)) return true;
    return false;
  }

  private boolean jj_3_1() {
    if (jj_scan_token(TOKEN_OPEN_BRACE)) return true;
    Token xsp;
    xsp = jj_scanpos;
    if (jj_3R_1()) {
    jj_scanpos = xsp;
    if (jj_3R_2()) {
    jj_scanpos = xsp;
    if (jj_3R_3()) {
    jj_scanpos = xsp;
    if (jj_3R_4()) {
    jj_scanpos = xsp;
    if (jj_3R_5()) {
    jj_scanpos = xsp;
    if (jj_3R_6()) {
    jj_scanpos = xsp;
    if (jj_3R_7()) {
    jj_scanpos = xsp;
    if (jj_3R_8()) {
    jj_scanpos = xsp;
    if (jj_3R_9()) return true;
    }
    }
    }
    }
    }
    }
    }
    }
    return false;
  }

  private boolean jj_3R_6() {
    if (jj_scan_token(TOKEN_OPTIONS)) return true;
    return false;
  }

  private boolean jj_3R_18() {
    if (jj_scan_token(TOKEN_MIN)) return true;
    return false;
  }

  private boolean jj_3R_10() {
    if (jj_scan_token(TOKEN_BINARY)) return true;
    return false;
  }

  private boolean jj_3R_17() {
    if (jj_scan_token(TOKEN_MAX)) return true;
    return false;
  }

  /** Generated Token Manager. */
  public JsonParserTokenManager token_source;
  JavaCharStream jj_input_stream;
  /** Current token. */
  public Token token;
  /** Next token. */
  public Token jj_nt;
  private int jj_ntk;
  private Token jj_scanpos, jj_lastpos;
  private int jj_la;
  private int jj_gen;
  final private int[] jj_la1 = new int[26];
  static private int[] jj_la1_0;
  static private int[] jj_la1_1;
  static private int[] jj_la1_2;
  static {
      jj_la1_init_0();
      jj_la1_init_1();
      jj_la1_init_2();
   }
   private static void jj_la1_init_0() {
      jj_la1_0 = new int[] {0x2800,0x10000000,0x10302f00,0x8000,0x10000000,0x8000,0x10200000,0x10200000,0x10200000,0x10000000,0x10000000,0x8000,0x0,0x0,0x0,0x10302f00,0x10200000,0x10200000,0x10200000,0x10000000,0x10000000,0x8000,0x0,0x0,0x0,0x10302f00,};
   }
   private static void jj_la1_init_1() {
      jj_la1_1 = new int[] {0x0,0x2000040,0x200ffc0,0x0,0x2000040,0x0,0x0,0x0,0x0,0x2000000,0x2000000,0x0,0x1ff0000,0x0,0x0,0x200ffc0,0x0,0x0,0x0,0x2000000,0x2000000,0x0,0x1ff0000,0x0,0x0,0x200ffc0,};
   }
   private static void jj_la1_init_2() {
      jj_la1_2 = new int[] {0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x20,0x20,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x20,0x20,0x0,};
   }
  final private JJCalls[] jj_2_rtns = new JJCalls[2];
  private boolean jj_rescan = false;
  private int jj_gc = 0;

  /** Constructor with InputStream. */
  public JsonParser(java.io.InputStream stream) {
     this(stream, null);
  }
  /** Constructor with InputStream and supplied encoding */
  public JsonParser(java.io.InputStream stream, String encoding) {
    try { jj_input_stream = new JavaCharStream(stream, encoding, 1, 1); } catch(java.io.UnsupportedEncodingException e) { throw new RuntimeException(e); }
    token_source = new JsonParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 26; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  /** Reinitialise. */
  public void ReInit(java.io.InputStream stream) {
     ReInit(stream, null);
  }
  /** Reinitialise. */
  public void ReInit(java.io.InputStream stream, String encoding) {
    try { jj_input_stream.ReInit(stream, encoding, 1, 1); } catch(java.io.UnsupportedEncodingException e) { throw new RuntimeException(e); }
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 26; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  /** Constructor. */
  public JsonParser(java.io.Reader stream) {
    jj_input_stream = new JavaCharStream(stream, 1, 1);
    token_source = new JsonParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 26; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  /** Reinitialise. */
  public void ReInit(java.io.Reader stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 26; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  /** Constructor with generated Token Manager. */
  public JsonParser(JsonParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 26; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  /** Reinitialise. */
  public void ReInit(JsonParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 26; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  private Token jj_consume_token(int kind) throws ParseException {
    Token oldToken;
    if ((oldToken = token).next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    if (token.kind == kind) {
      jj_gen++;
      if (++jj_gc > 100) {
        jj_gc = 0;
        for (int i = 0; i < jj_2_rtns.length; i++) {
          JJCalls c = jj_2_rtns[i];
          while (c != null) {
            if (c.gen < jj_gen) c.first = null;
            c = c.next;
          }
        }
      }
      return token;
    }
    token = oldToken;
    jj_kind = kind;
    throw generateParseException();
  }

  static private final class LookaheadSuccess extends java.lang.Error { }
  final private LookaheadSuccess jj_ls = new LookaheadSuccess();
  private boolean jj_scan_token(int kind) {
    if (jj_scanpos == jj_lastpos) {
      jj_la--;
      if (jj_scanpos.next == null) {
        jj_lastpos = jj_scanpos = jj_scanpos.next = token_source.getNextToken();
      } else {
        jj_lastpos = jj_scanpos = jj_scanpos.next;
      }
    } else {
      jj_scanpos = jj_scanpos.next;
    }
    if (jj_rescan) {
      int i = 0; Token tok = token;
      while (tok != null && tok != jj_scanpos) { i++; tok = tok.next; }
      if (tok != null) jj_add_error_token(kind, i);
    }
    if (jj_scanpos.kind != kind) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) throw jj_ls;
    return false;
  }


/** Get the next Token. */
  final public Token getNextToken() {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    jj_gen++;
    return token;
  }

/** Get the specific Token. */
  final public Token getToken(int index) {
    Token t = token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  private int jj_ntk() {
    if ((jj_nt=token.next) == null)
      return (jj_ntk = (token.next=token_source.getNextToken()).kind);
    else
      return (jj_ntk = jj_nt.kind);
  }

  private java.util.List<int[]> jj_expentries = new java.util.ArrayList<int[]>();
  private int[] jj_expentry;
  private int jj_kind = -1;
  private int[] jj_lasttokens = new int[100];
  private int jj_endpos;

  private void jj_add_error_token(int kind, int pos) {
    if (pos >= 100) return;
    if (pos == jj_endpos + 1) {
      jj_lasttokens[jj_endpos++] = kind;
    } else if (jj_endpos != 0) {
      jj_expentry = new int[jj_endpos];
      for (int i = 0; i < jj_endpos; i++) {
        jj_expentry[i] = jj_lasttokens[i];
      }
      jj_entries_loop: for (java.util.Iterator<?> it = jj_expentries.iterator(); it.hasNext();) {
        int[] oldentry = (int[])(it.next());
        if (oldentry.length == jj_expentry.length) {
          for (int i = 0; i < jj_expentry.length; i++) {
            if (oldentry[i] != jj_expentry[i]) {
              continue jj_entries_loop;
            }
          }
          jj_expentries.add(jj_expentry);
          break jj_entries_loop;
        }
      }
      if (pos != 0) jj_lasttokens[(jj_endpos = pos) - 1] = kind;
    }
  }

  /** Generate ParseException. */
  public ParseException generateParseException() {
    jj_expentries.clear();
    boolean[] la1tokens = new boolean[71];
    if (jj_kind >= 0) {
      la1tokens[jj_kind] = true;
      jj_kind = -1;
    }
    for (int i = 0; i < 26; i++) {
      if (jj_la1[i] == jj_gen) {
        for (int j = 0; j < 32; j++) {
          if ((jj_la1_0[i] & (1<<j)) != 0) {
            la1tokens[j] = true;
          }
          if ((jj_la1_1[i] & (1<<j)) != 0) {
            la1tokens[32+j] = true;
          }
          if ((jj_la1_2[i] & (1<<j)) != 0) {
            la1tokens[64+j] = true;
          }
        }
      }
    }
    for (int i = 0; i < 71; i++) {
      if (la1tokens[i]) {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.add(jj_expentry);
      }
    }
    jj_endpos = 0;
    jj_rescan_token();
    jj_add_error_token(0, 0);
    int[][] exptokseq = new int[jj_expentries.size()][];
    for (int i = 0; i < jj_expentries.size(); i++) {
      exptokseq[i] = jj_expentries.get(i);
    }
    return new ParseException(token, exptokseq, tokenImage);
  }

  /** Enable tracing. */
  final public void enable_tracing() {
  }

  /** Disable tracing. */
  final public void disable_tracing() {
  }

  private void jj_rescan_token() {
    jj_rescan = true;
    for (int i = 0; i < 2; i++) {
    try {
      JJCalls p = jj_2_rtns[i];
      do {
        if (p.gen > jj_gen) {
          jj_la = p.arg; jj_lastpos = jj_scanpos = p.first;
          switch (i) {
            case 0: jj_3_1(); break;
            case 1: jj_3_2(); break;
          }
        }
        p = p.next;
      } while (p != null);
      } catch(LookaheadSuccess ls) { }
    }
    jj_rescan = false;
  }

  private void jj_save(int index, int xla) {
    JJCalls p = jj_2_rtns[index];
    while (p.gen > jj_gen) {
      if (p.next == null) { p = p.next = new JJCalls(); break; }
      p = p.next;
    }
    p.gen = jj_gen + xla - jj_la; p.first = token; p.arg = xla;
  }

  static final class JJCalls {
    int gen;
    Token first;
    int arg;
    JJCalls next;
  }

}
